/**
 * Copyright (c) 2006 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * History:
 *
 * $Log: AtgLangZeitDatenFahrzeugAnzahl.java,v $
 * Revision 1.1  2007/07/04 11:42:23  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.3  2006/11/10 10:41:17  Strey
 * diverse private static Strings auf public gesetzt
 *
 * Revision 1.2  2006/08/29 08:54:21  Schawaller
 * neue methode: alle ATL als Liste
 *
 * Revision 1.1  2006/08/28 13:43:42  Schawaller
 * Neue klasse zum kapseln von ATG
 *
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 *
 * Die Klasse kapselt die Attributgruppe
 * <code>atg.langZeitDatenFahrzeugAnzahl</code>.
 *
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 *
 * <p>
 * Organisation: Logos GmbH Rostock (www.logos-ib.de)
 * </p>
 *
 * @author J.Schawaller
 * @version 1.0
 */
public class AtgLangZeitDatenFahrzeugAnzahl {

	/**
	 * _Data - Daten
	 */
	private final Data _Data;

	/**
	 * __TYP_ATG_LangZeitDatenFahrzeugAnzahl - Attributgruppe
	 */
	private static AttributeGroup __TYP_ATG_LangZeitDatenFahrzeugAnzahl;

	/**
	 * ATG_PID_LangZeitDatenFahrzeugAnzahl - Name der PID
	 */
	public static String ATG_PID_LangZeitDatenFahrzeugAnzahl = "atg.langZeitDatenFahrzeugAnzahl";

	/**
	 * ATT_TYP_LangzeitdatenFahrzeugAnzahl - Namen der Attribut-Typen
	 */
	private static String ATT_TYP_LangzeitdatenFahrzeugAnzahl = "LangzeitdatenFahrzeuganzahl";

	public static final String ATTR_NAME_KFZ = "Kfz";
	public static final String ATTR_NAME_PKWAE = "PkwÄ";
	public static final String ATTR_NAME_LKWAE = "LkwÄ";
	public static final String ATTR_NAME_KFZNK = "KfzNk";
	public static final String ATTR_NAME_PKWG = "PkwG";
	public static final String ATTR_NAME_PKW = "Pkw";
	public static final String ATTR_NAME_KRAD = "Krad";
	public static final String ATTR_NAME_LFW = "Lfw";
	public static final String ATTR_NAME_PKWA = "PkwA";
	public static final String ATTR_NAME_LKW = "Lkw";
	public static final String ATTR_NAME_BUS = "Bus";
	public static final String ATTR_NAME_LKWK = "LkwK";
	public static final String ATTR_NAME_LKWA = "LkwA";
	public static final String ATTR_NAME_SATTELKFZ = "SattelKfz";

	/**
	 * __ATTR_NAMEN - Namen der Attribute als Array zusammengefa&szlig;t
	 */
	public static final String[] ATTR_NAMEN = {
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KFZ,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWAE,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWAE,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KFZNK,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWG,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKW,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KRAD,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LFW,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWA,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKW,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_BUS,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWK,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWA,
		AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_SATTELKFZ };

	/**
	 * Erzeugt eine neue Instanz mit den übergebenen Daten.
	 *
	 * @param data
	 *            Data
	 */
	public AtgLangZeitDatenFahrzeugAnzahl(final Data data) {
		this._Data = data;
	}

	/**
	 * Erzeugt eine neue Instanz und erstellt ein Data.
	 *
	 */
	public AtgLangZeitDatenFahrzeugAnzahl() {
		if (AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl == null) {
			AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl = LogosClientDavConnection
					.getInstance()
					.getDataModel()
					.getAttributeGroup(
							AtgLangZeitDatenFahrzeugAnzahl.ATG_PID_LangZeitDatenFahrzeugAnzahl);
		}
		_Data = LogosClientDavConnection
				.getInstance()
				.createData(
						AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl);
	}

	/**
	 * Initialisiert die Werte des Data für den generichen Zugriff.
	 *
	 */
	public void doInitialize() {
		if (AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl == null) {
			AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl = LogosClientDavConnection
					.getInstance()
					.getDataModel()
					.getAttributeGroup(
							AtgLangZeitDatenFahrzeugAnzahl.ATG_PID_LangZeitDatenFahrzeugAnzahl);
		}
		final Iterator attrIterator = AtgLangZeitDatenFahrzeugAnzahl.__TYP_ATG_LangZeitDatenFahrzeugAnzahl
				.getAttributes().iterator();
		while (attrIterator.hasNext()) {
			final Attribute oAttribute = (Attribute) attrIterator.next();
			if (AtgLangZeitDatenFahrzeugAnzahl.ATT_TYP_LangzeitdatenFahrzeugAnzahl
					.equals(oAttribute.getAttributeType().getName())) {
				new AtlLangZeitDatenFahrzeugAnzahl(_Data.getItem(oAttribute
						.getName())).doInitialize();
			}
		} // end while
	}

	/**
	 * Gibt das Data wieder.
	 *
	 * @return Data
	 */
	public Data getData() {
		return _Data;
	}

	/*
	 * #### GETTER - Methoden für den Zugriff
	 */
	/**
	 * Gibt die Daten aller AttributListen als Liste wieder.
	 *
	 * @return List Daten aller AttributListen
	 */
	public List<AtlLangZeitDatenFahrzeugAnzahl> getAttributListenLangZeitDatenFahrzeugAnzahl() {
		final List result = new ArrayList<AtlLangZeitDatenFahrzeugAnzahl>(
				AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAMEN.length);
		for (final String attrName : AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAMEN) {
			result.add(new AtlLangZeitDatenFahrzeugAnzahl(_Data
					.getItem(attrName)));
		}
		return result;
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKfz() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KFZ));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwAE() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWAE));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKfzNk() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KFZNK));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwG() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWG));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkw() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKW));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getKrad() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_KRAD));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLfw() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LFW));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwAE() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWAE));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getPkwA() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_PKWA));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkw() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKW));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getBus() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_BUS));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwK() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWK));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getLkwA() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_LKWA));
	}

	public AtlLangZeitDatenFahrzeugAnzahl getSattelKfz() {
		return new AtlLangZeitDatenFahrzeugAnzahl(
				_Data.getItem(AtgLangZeitDatenFahrzeugAnzahl.ATTR_NAME_SATTELKFZ));
	}

	/**
	 * Gibt die Eigenschaften der Klasse als String wieder.
	 *
	 * @return String Eigenschaften der Klasse
	 */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("atg[")
		.append(AtgLangZeitDatenFahrzeugAnzahl.ATG_PID_LangZeitDatenFahrzeugAnzahl)
		.append("]: ");
		sb.append(" data[").append(_Data).append("]");
		return sb.toString();
	}

}
