/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/**
 * History:
 *
 * $Log: AtlGuete.java,v $
 * Revision 1.2  2007/08/22 07:48:44  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:42:57  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.3  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:18  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */

/**
 * kapselt die AttributListe atl.güte.
 */
public class AtlGuete {

	private final Data _Data;

	public static final String ATTR_NAME_INDEX = "Index";
	public static final String ATTR_NAME_VERFAHREN = "Verfahren";

	public static final String ZUSTAND_INDEX_NICHT_ERMITTELBAR = "nicht ermittelbar";
	public static final String ZUSTAND_INDEX_FEHLERHAFT = "fehlerhaft";
	public static final String ZUSTAND_INDEX_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";
	public static final String ZUSTAND_VERFAHREN_STANDARD = "Standard";

	public AtlGuete(final Data data) {
		_Data = data;
	}

	protected void doInitialize() {
		setIndex((byte) 1);
		_Data.getScaledValue("Verfahren").setText(
				AtlGuete.ZUSTAND_VERFAHREN_STANDARD);
	}

	public IntegerValueRange getRangeOfGueteIndex() {
		return ValueRangeHashtable.getInstance(this._Data).get("att.güteIndex");
	}

	public IntegerValueRange getRangeOfGueteVerfahren() {
		return ValueRangeHashtable.getInstance(this._Data).get(
				"att.güteVerfahren");
	}

	public double getIndex() {
		return _Data.getScaledValue("Index").doubleValue();
	}

	public void setIndex(final byte index) {
		_Data.getScaledValue("Index").set(index);
	}

	public IntegerValueState getZustandIndex() {
		return _Data.getScaledValue("Index").getState();
	}

	public boolean isState() {
		return _Data.getUnscaledValue("Index").isState();
	}

	public boolean isNumber() {
		return !isState();
	}

	public void setZustandIndex(final IntegerValueState integerValueState) {
		_Data.getScaledValue("Index").setState(integerValueState);
	}

	public void setZustandIndex(final String zustandsName) {
		_Data.getScaledValue("Index").setText(zustandsName);
	}

	public byte getVerfahren() {
		return _Data.getScaledValue("Verfahren").byteValue();
	}

	public void setVerfahren(final byte verfahren) {
		_Data.getScaledValue("Verfahren").set(verfahren);
	}

	public IntegerValueState getZustandVerfahren() {
		return _Data.getScaledValue("Verfahren").getState();
	}

	public void setZustandVerfahren(final IntegerValueState integerValueState) {
		_Data.getScaledValue("Verfahren").setState(integerValueState);
	}

	public void setZustandVerfahren(final String zustandsName) {
		_Data.getScaledValue("Verfahren").setText(zustandsName);
	}

}
