/**
 * Copyright (c) 2006 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * History:
 * 2007-07-13 [KS]  Bugfix wegen Zugriffsfehler in Kernsoftware beim Aufruf von
 *                  Data.getScaledValue(String).shortValue() und Skalierung !=1
 *
 * $Log: AtlLangZeitDatenGeschwindigkeit.java,v $
 * Revision 1.2  2007/07/13 11:31:22  Strey
 * Bugfix wegen Zugriffsfehler in Kernsoftware beim Zugriff auf Data.getScaledValue().shortValue().
 *
 * Revision 1.1  2007/07/04 11:43:03  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.1  2006/08/28 13:44:06  Schawaller
 * Neue klasse zum kapseln von ATL
 *
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/**
 *
 * Die Klasse kapselt die Attribut-Liste
 * <code>atl.langZeitDatenGeschwindigkeit</code>.
 *
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 *
 * <p>
 * Organisation: Logos GmbH Rostock (www.logos-ib.de)
 * </p>
 *
 * @author J.Schawaller
 * @version 1.0
 */
public class AtlLangZeitDatenGeschwindigkeit {

	private final Data _Data;

	public static final String ATTR_NAME_WERT = "Wert";
	public static final String ATTR_NAME_GUETE = "Güte";

	private static final String TYP_ATT_LANG_ZEIT_DATEN_GESCHWINDIGKEIT = "att.langZeitDatenGeschwindigkeit";

	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
	public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

	public AtlLangZeitDatenGeschwindigkeit(final Data data) {
		_Data = data;
	}

	public void doInitialize() {
		new AtlGuete(
				_Data.getItem(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_GUETE))
		.doInitialize();
		_Data.getScaledValue(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT)
		.setText(
				AtlLangZeitDatenGeschwindigkeit.ZUSTAND_WERT_NICHT_ERMITTELBAR);
	}

	public IntegerValueRange getRangeOfGeschwindigkeit() {
		return ValueRangeHashtable
				.getInstance(this._Data)
				.get(AtlLangZeitDatenGeschwindigkeit.TYP_ATT_LANG_ZEIT_DATEN_GESCHWINDIGKEIT);
	}

	public short getWert() {
		// BUGFIX wegen Exception mit neuer Kernsoftware
		// return _Data.getScaledValue(ATTR_NAME_WERT).shortValue();
		return (short) (_Data
				.getScaledValue(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT)
				.doubleValue());
	}

	public void setWert(final short wert) {
		if (wert < 0) {
			_Data.getUnscaledValue(
					AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT).set(wert);
		} else {
			_Data.getScaledValue(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT)
			.set(wert);
		}
	}

	public IntegerValueState getZustandWert() {
		return _Data.getScaledValue(
				AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT).getState();
	}

	public void setZustandWert(final IntegerValueState integerValueState) {
		_Data.getScaledValue(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT)
		.setState(integerValueState);
	}

	public void setZustandWert(final String zustandsName) {
		_Data.getScaledValue(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT)
		.setText(zustandsName);
	}

	public AtlGuete getGuete() {
		return new AtlGuete(
				_Data.getItem(AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_GUETE));
	}

	public boolean isState() {
		return _Data.getUnscaledValue(
				AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT).isState();
	}

	public boolean isNumber() {
		return _Data.getUnscaledValue(
				AtlLangZeitDatenGeschwindigkeit.ATTR_NAME_WERT).isNumber();
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("atl[")
		.append(AtlLangZeitDatenGeschwindigkeit.TYP_ATT_LANG_ZEIT_DATEN_GESCHWINDIGKEIT)
		.append("]: ");
		sb.append(" data[").append(_Data).append("]");
		return sb.toString();
	}

}
