/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * Projektbeschreibung:
 * @title
 * @version 1.0
 * @copyright Copyright (c) 2004
 */

/**
 * History:
 *
 * $Log: Bundesland.java,v $
 * Revision 1.1  2007/07/04 11:43:36  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.4  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.3  2005/09/22 06:55:22  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:25:50  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.HashMap;
import java.util.TreeMap;

/**
 * kapselt die Bundesl&auml;nder, die einer LangzeitZ&auml;hlStelle zu geordnet
 * ist und deren Code durch die BASt vorgegeben ist.
 *
 */
public class Bundesland implements Comparable {

	private static HashMap<Integer, Bundesland> __HashMapCodeAsKey;

	public static final Bundesland SCHLESWIG_HOLSTEIN = new Bundesland(
			"Schleswig-Holstein", 1, "SH");
	public static final Bundesland HAMBURG = new Bundesland("Hamburg", 2, "HH");
	public static final Bundesland NIEDERSACHSEN = new Bundesland(
			"Niedersachsen", 3, "NI");
	public static final Bundesland BREMEN = new Bundesland("Bremen", 4, "HB");
	public static final Bundesland NORDRHEIN_WESTFALEN = new Bundesland(
			"Nordrhein-Westfalen", 5, "NW");
	public static final Bundesland HESSEN = new Bundesland("Hessen", 6, "HE");
	public static final Bundesland RHEINLAND_PFALZ = new Bundesland(
			"Rheinland-Pfalz", 7, "RP");
	public static final Bundesland BADEN_WUERTTEMBERG = new Bundesland(
			"Baden-Würtenberg", 8, "BW");
	public static final Bundesland BAYERN = new Bundesland("Bayern", 9, "BY");
	public static final Bundesland SAARLAND = new Bundesland("Saarland", 10,
			"SL");
	public static final Bundesland BERLIN = new Bundesland("Berlin", 11, "BE");
	public static final Bundesland BRANDENBURG = new Bundesland("Brandenburg",
			12, "BB");
	public static final Bundesland MECKLENBURG_VORPOMMERN = new Bundesland(
			"Mecklenburg-Vorpommern", 13, "MV");
	public static final Bundesland SACHSEN = new Bundesland("Sachsen", 14, "SA");
	public static final Bundesland SACHSEN_ANHALT = new Bundesland(
			"Sachsen-Anhalt", 15, "ST");
	public static final Bundesland THUERINGEN = new Bundesland("Thüringen", 16,
			"TH");

	private final String _Name;
	private final int _Code;
	private final String _Abbreviation;

	private static Bundesland[] __BundesLaenderArray;

	private Bundesland(final String name, final int code,
			final String abbreviation) {
		_Name = name;
		_Code = code;
		_Abbreviation = abbreviation;
		if (Bundesland.__HashMapCodeAsKey == null) {
			Bundesland.__HashMapCodeAsKey = new HashMap<Integer, Bundesland>();
		}
		Bundesland.__HashMapCodeAsKey.put(code, this);
	}

	public static Bundesland getInstanceByCode(final int code) {
		return Bundesland.__HashMapCodeAsKey.get(code);
	}

	public Bundesland[] getLaender() {
		if (Bundesland.__BundesLaenderArray == null) {
			final TreeMap<Integer, Bundesland> map = new TreeMap<Integer, Bundesland>(
					Bundesland.__HashMapCodeAsKey);
			Bundesland.__BundesLaenderArray = map.values().toArray(
					new Bundesland[map.size()]);
		}
		return Bundesland.__BundesLaenderArray;
	}

	@Override
	public String toString() {
		return _Name;
	}

	public String getAbbreviation() {
		return _Abbreviation;
	}

	public int getKennung() {
		return _Code;
	}

	public String getKennungAsString() {
		if (_Code < 10) {
			return '0' + String.valueOf(_Code);
		}
		return String.valueOf(_Code);
	}

	@Override
	public int compareTo(final Object o) {
		return ((Bundesland) o)._Name.compareTo(_Name);
	}

}
