/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * Projektbeschreibung:
 * @title
 * @version 1.0
 * @copyright Copyright (c) 2004
 */

/**
 * History:
 *
 * $Log: Fahrstreifen.java,v $
 * Revision 1.1  2007/07/04 11:43:46  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.11  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.10  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.9  2005/09/22 06:55:22  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.8  2005/09/16 08:18:14  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.7  2005/09/09 07:37:28  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.6  2005/09/08 11:02:38  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2005/09/07 06:04:36  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/06 13:59:17  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/09/06 12:40:18  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:18  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * kapselt Instancen des DatenModellTypes fahrStreifen.
 *
 * <p>
 * <b>Achtung:</b>
 *
 * Das Attribute "Typ" wird zur Sortierung der Fahrstreifen benutzt. Es wird
 * davon ausgegangen, dass die Zust&auml;nde/Werte des Attributtypes
 * att.fahrStreifenTyp von aussen nach innen aufsteigen.
 * </p>
 */
public class Fahrstreifen extends ConfigurationObjectWrapper {

	private static final Debug LOGGER = Debug.getLogger();

	private static WeakHashMap<String, Fahrstreifen> __Cache = new WeakHashMap<String, Fahrstreifen>();
	private static ConfigurationObjectType __ConfigurationObjectType;

	private static ConfigurationObjectType __ConfigurationObjectTypeFahrstreifenQuelleASCII;
	private static ConfigurationObjectType __ConfigurationObjectTypeFahrstreifenQuelleVirtuell;
	private static ConfigurationObjectType __ConfigurationObjectTypeDeLve;
	private static ConfigurationObjectType __ConfigurationObjectTypeDeAxl;

	private static AttributeGroup __AtgFahrStreifen;

	private String _Typ;
	private int _TypNr;

	private String _Lage;
	private int _LageNr;

	private String _ErsatzFahrstreifenPID;
	private FahrstreifenQuelle _FahrstreifenQuelle;
	private DEKanal.Axl _DEKanalAxl;

	private MessQuerschnitt _MessQuerschnitt;

	private ConfigurationObject _ConfigurationObject;

	/**
	 * <p>
	 * setzt den MessQuerschnitt, zu dem der fahrstreifen geh&ouml;rt.
	 * </p>
	 *
	 * @param messQuerschnitt
	 *            MessQuerschnitt
	 */
	protected void setMessQuerschnitt(final MessQuerschnitt messQuerschnitt) {
		this._MessQuerschnitt = messQuerschnitt;
	}

	/**
	 * <p>
	 * gibt den MessQuerschnitt zur&uuml;ck, zu dem der Fahrstreifen
	 * geh&ouml;rt, wenn dieser irgenwann gesetzt worden ist.
	 * </p>
	 *
	 * @return MessQuerschnitt
	 */
	public MessQuerschnitt getMessQuerschnitt() {
		return _MessQuerschnitt;
	}

	/**
	 * gibt ein Object auf die FahrstreifenQuelle zurück, das eine Instance der
	 * Klassen FahrstreifenQuelleASCII, FahrstreifenQuelleDeLve oder
	 * FahrstreifenQuelleVirtuell ist.
	 *
	 * @return FahrstreifenQuelle
	 */
	public FahrstreifenQuelle getFahrstreifenQuelle() {
		return _FahrstreifenQuelle;
	}

	/**
	 * gibt ein Object auf den DEKanal.Axl zurück, wenn f&uuml;r den
	 * Fahrstreifen Achslasten erfa&szlig;t werden..
	 *
	 * @return DEKanal f&uuml;r Achslastdatenerfassung oder null, wenn keine
	 *         Erfassung.
	 */
	public DEKanal.Axl getDEKanalAxl() {
		if (AxlLastMessStelle.isNotInitialized()) {
			try {
				AxlLastMessStelle.getAll();
			} catch (final Exception e) {
				Fahrstreifen.LOGGER.error("Fehler beim Laden der AchslastMessStellen", e);
			}
		}
		return _DEKanalAxl;
	}

	/**
	 * setzt den DEKanal.Axl.
	 *
	 * @param deKanal
	 *            der Kanal
	 */
	public void setDEKanalAxl(final DEKanal.Axl deKanal) {
		_DEKanalAxl = deKanal;
	}

	/**
	 * gibt true zur&uuml;ck, wenn die Quelle vom DEKanal.Axl ist.
	 *
	 * @return boolean
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	public boolean isAchslastErfassung() throws DataModellMismatchException {
		if (AxlLastMessStelle.isNotInitialized()) {
			try {
				AxlLastMessStelle.getAll();
			} catch (final Exception e) {
				Fahrstreifen.LOGGER.error("Fehler beim Laden der AchslastMessStellen", e);
			}
		}
		return (_DEKanalAxl != null);
	}

	/**
	 * gibt den ErsatzFahrstreifen zurück oder null, wenn keiner definiert
	 * wurde.
	 *
	 * @throws DataModellObjectNotFoundException
	 * @throws DataModellMismatchException
	 * @return Fahrstreifen
	 * @throws DataModellObjectNotFoundException
	 *             ungültiges Datenmodell
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	public Fahrstreifen getErsatzFahrstreifen() throws DataModellObjectNotFoundException, DataModellMismatchException {

		if (_ErsatzFahrstreifenPID != null) {
			return Fahrstreifen.getInstance(_ErsatzFahrstreifenPID);
		} else {
			return null;
		}
	}

	/**
	 * gibt den Typ des Fahrstreifen zurück: SonstigerFahrStreifen, Kriechspur,
	 * HFS, 1ÜFS, 2ÜFS usw..
	 *
	 * @return String
	 */
	public String getTyp() {
		return _Typ;
	}

	private Fahrstreifen(final String pid) throws DataModellObjectNotFoundException, DataModellMismatchException {
		super(pid);
	}

	private Fahrstreifen(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject fahrstreifen) throws DataModellMismatchException {

		final Data oData = fahrstreifen.getConfigurationData(Fahrstreifen.__AtgFahrStreifen);
		if (oData != null) {

			if (oData.getUnscaledValue("Typ").isState()) {
				_Typ = oData.getUnscaledValue("Typ").getState().getName();
				_TypNr = oData.getUnscaledValue("Typ").intValue();
			}

			if (oData.getUnscaledValue("Lage").isState()) {
				_Lage = oData.getUnscaledValue("Lage").getState().getName();
			} else {
				_Lage = oData.getUnscaledValue("Lage").getValueText();
			}
			_LageNr = oData.getUnscaledValue("Lage").intValue();

			if (oData.getReferenceValue("ErsatzFahrStreifen").getSystemObject() != null) {
				_ErsatzFahrstreifenPID = oData.getReferenceValue("ErsatzFahrStreifen").getSystemObject().getPid();
			}

			final SystemObject obj = oData.getReferenceValue("FahrStreifenQuelle").getSystemObject();
			if (obj != null) {
				if (obj instanceof ConfigurationObject) {
					_FahrstreifenQuelle = Fahrstreifen.getFahrstreifenquelle((ConfigurationObject) obj);
					if (_FahrstreifenQuelle instanceof DEKanal.Lve) {
						((DEKanal.Lve) _FahrstreifenQuelle).setFahrstreifen(this);
					}
					if (_FahrstreifenQuelle instanceof DEKanal.Axl) {
						((DEKanal.Axl) _FahrstreifenQuelle).setFahrstreifen(this);
					}
				} else {
					throw new DataModellMismatchException("Die ObjektReferenz FahrstreifenQuelle des Fahrstreifen \""
							+ getPID() + "\" verweist nicht auf ein ConfigurationsObject", getPID());
				}
			} else {
				// throw new DataModellMismatchException("Fahrstreifen \"" +
				// getPID() +
				// "\" keine ObjektReferenz auf eine FahrstreifenQuelle",
				// getPID());
			}
		}
		Fahrstreifen.__Cache.put(this.getPID(), this);
	}

	/**
	 * gibt die Instance der FahrStreifen mit der PID zurück. Es wird eine
	 * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
	 * ist oder das zur PID gehörende Objekt nich vom typ fahrStreifen ist. Eine
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
	 * falsche Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellObjectNotFoundException
	 *             ungültiges Datenmodell
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 * @return Fahrstreifen
	 */
	public static Fahrstreifen getInstance(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {

		Fahrstreifen retValue = Fahrstreifen.__Cache.get(pid);
		if (retValue == null) {
			retValue = new Fahrstreifen(pid);
		}
		return retValue;
	}

	protected static Fahrstreifen getInstance(final ConfigurationObject fahrStreifen)
			throws DataModellMismatchException {

		Fahrstreifen retValue = Fahrstreifen.__Cache.get(fahrStreifen.getPid());
		if (retValue == null) {
			retValue = new Fahrstreifen(fahrStreifen);
		}
		return retValue;
	}

	/**
	 * gibt die DatenModellObjekte vom typ fahrstreifen zurück, wenn keiner
	 * dieser Objekte Fehler in den ObjektReferenzen aufweist.
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
	 * falsche Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 * @return Fahrstreifen[]
	 */
	public static Fahrstreifen[] getAll() throws DataModellMismatchException {

		final List list = LogosClientDavConnection.getInstance().getDataModel().getType("typ.fahrStreifen")
				.getElements();
		final Fahrstreifen[] retValue = new Fahrstreifen[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new Fahrstreifen((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("Fahrstreifen - PID: \"").append(getPID()).append("\"  ");
		sb.append("Name: \"").append(getName());
		return sb.toString();
	}

	public String getMultilineRepresentation(final int indent) {
		final String lf = System.getProperty("line.separator");
		final StringBuffer sIndent = new StringBuffer();
		for (int i = 0; i < indent; i++) {
			sIndent.append(' ');
		}
		final StringBuffer sb = new StringBuffer();
		sb.append(sIndent).append("PID: \"").append(getPID()).append("\"").append(lf);
		sb.append(sIndent).append("Name: \"").append(getName()).append("\"").append(lf);
		sb.append(sIndent).append("Typ: \"").append(_Typ).append("\"").append(lf);
		sb.append(sIndent).append("ErsatzFahrstreifenPID: \"").append(_ErsatzFahrstreifenPID).append("\"").append(lf);
		if (_FahrstreifenQuelle != null) {
			sb.append(sIndent).append("FahrstreifenQuelle: \"").append(_FahrstreifenQuelle).append("\"");
		} else {
			sb.append(sIndent).append("FahrstreifenQuelle ist null");
		}
		return sb.toString();
	}

	public String getMultilineRepresentation() {
		return getMultilineRepresentation(0);
	}

	@Override
	ConfigurationObjectType getType() {
		if (Fahrstreifen.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			Fahrstreifen.__ConfigurationObjectType = (ConfigurationObjectType) model.getType("typ.fahrStreifen");
			Fahrstreifen.__AtgFahrStreifen = model.getAttributeGroup("atg.fahrStreifen");
			final List<SystemObject> list = Fahrstreifen.__ConfigurationObjectType.getElements();
			model.getConfigurationData(list, Fahrstreifen.__AtgFahrStreifen);
		}
		return Fahrstreifen.__ConfigurationObjectType;
	}

	public String getLage() {
		return _Lage;
	}

	@Override
	public String getTreeString() {
		String s;
		if (_LageNr < 0) {
			s = _Typ + " " + String.valueOf(Math.abs(_LageNr)) + ". FS rechts vom HFS";
		} else {
			s = _Lage;
		}
		if (_FahrstreifenQuelle == null) {
			return s + " (ohne Datenerfassung)";
		} else {
			return s;
		}
	}

	private static FahrstreifenQuelle getFahrstreifenquelle(final ConfigurationObject confObj)
			throws DataModellMismatchException {

		if (Fahrstreifen.__ConfigurationObjectTypeDeLve == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			Fahrstreifen.__ConfigurationObjectTypeDeLve = (ConfigurationObjectType) model.getType("typ.deLve");
			Fahrstreifen.__ConfigurationObjectTypeDeAxl = (ConfigurationObjectType) model.getType("typ.deAxl");
			Fahrstreifen.__ConfigurationObjectTypeFahrstreifenQuelleASCII = (ConfigurationObjectType) model
					.getType("typ.verkehrsDatenQuelleASCII");
			Fahrstreifen.__ConfigurationObjectTypeFahrstreifenQuelleVirtuell = (ConfigurationObjectType) model
					.getType("typ.verkehrsDatenQuelleVirtuell");
		}

		if (confObj != null) {
			if (confObj.isOfType(Fahrstreifen.__ConfigurationObjectTypeDeLve)) {
				return (FahrstreifenQuelle) DEKanal.getInstance(confObj);
			}
			if (confObj.isOfType(Fahrstreifen.__ConfigurationObjectTypeFahrstreifenQuelleASCII)) {
				return new FahrstreifenQuelleASCII(confObj);
			}
			if (confObj.isOfType(Fahrstreifen.__ConfigurationObjectTypeFahrstreifenQuelleVirtuell)) {
				return new FahrstreifenQuelleVirtuell(confObj);
			}
			if (confObj.isOfType(Fahrstreifen.__ConfigurationObjectTypeDeAxl)) {
				return (FahrstreifenQuelle) DEKanal.getInstance(confObj);
			}
		}
		return null;
	}

	static class FSComperatorVonAussenNachInnen implements java.util.Comparator<Fahrstreifen> {
		FSComperatorVonAussenNachInnen() {
		}

		@Override
		public boolean equals(final Object obj) {
			return (this == obj);
		}

		@Override
		public int compare(final Fahrstreifen fs01, final Fahrstreifen fs02) {
			int retValue = 0;
			if (fs01 != fs02) {
				retValue = (fs01._LageNr - fs02._LageNr);
			}
			return retValue;
		}
	}

}
