/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: Unterzentrale.java,v $
 * Revision 1.2  2007/07/25 13:02:08  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:44:22  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.4  2006/04/28 05:48:50  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.3  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/03/16 16:08:36  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:26:12  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp uz (Unterzentrale)
 */
public class Unterzentrale extends ConfigurationObjectWrapper {

	private static WeakHashMap<String, Unterzentrale> __Cache = new WeakHashMap<String, Unterzentrale>();

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static ConfigurationObjectType __ConfigurationObjectTypeOfKRI;
	private static AttributeGroup __AtgGeraet;

	private String _Bezeichnung;
	private int _KnotenNummer;
	private String _UmsetzungsModul;

	private ArrayList<KRI> _KRIList;

	/**
	 * gibt die Bezeichnung des EAK zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getBezeichnung() {
		return _Bezeichnung;
	}

	/**
	 * gibt die Knotennummer (OSI-7-Adresse) des Ger&auml;tes zur&uuml;ck.
	 *
	 * @return die Knotennummer;
	 */
	public int getKnotenNummer() {
		return _KnotenNummer;
	}

	/**
	 * gibt das für die Umsetzung des Protokolls (Lesen und Schreiben)
	 * verwendete Modul an diesem Knoten zur&uuml;ck.
	 *
	 * @return Klassennamen des UmsetzungsModuls.
	 */
	public String getUmsetzungsModul() {
		return _UmsetzungsModul;
	}

	/**
	 * gibt die KRI's zur&uuml;ck oder null, wenn keine angeschlossen sind.
	 *
	 * @return SteuerModule.
	 */
	public KRI[] getKRIs() {
		if ((_KRIList != null) && (_KRIList.size() > 0)) {
			return _KRIList.toArray(new KRI[_KRIList.size()]);
		} else {
			return null;
		}
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das Object mit der angegebenen Pid
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
	 *
	 * @param pid
	 *            String
	 * @throws DataModellMismatchException
	 * @throws DataModellObjectNotFoundException
	 */
	protected Unterzentrale(final String pid)
			throws DataModellMismatchException,
			DataModellObjectNotFoundException {
		super(pid);
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das angegebene Object.
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(ConfigurationObject
	 *      confObj)
	 *
	 * @param confObj
	 *            ConfigurationObject
	 * @throws DataModellMismatchException
	 */
	protected Unterzentrale(final ConfigurationObject confObj)
			throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj)
			throws DataModellMismatchException {

		// if (__AtgGeraet == null) {
		// DataModel model =
		// LogosClientDavConnection.getInstance().getDataModel();
		// __ConfigurationObjectTypeOfKRI = (ConfigurationObjectType)
		// model.getType("typ.kri");
		// __AtgGeraet = model.getAttributeGroup("atg.gerät");
		// }

		final Data oData = confObj
				.getConfigurationData(Unterzentrale.__AtgGeraet);
		if (oData != null) {
			_Bezeichnung = oData.getTextValue("Bezeichnung").getText();
			_KnotenNummer = oData.getUnscaledValue("KnotenNummer").intValue();
			_UmsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
		}

		final ObjectSet os = confObj.getObjectSet("AnschlussPunkteGerät");
		if (os != null) {
			final List list = ConfigurationObjectWrapper
					.getKommunikationPartner(os.getElements());
			if ((list != null) && (list.size() > 0)) {
				_KRIList = new ArrayList<KRI>(list.size());
				for (int i = 0; i < list.size(); i++) {
					final ConfigurationObject kriObj = (ConfigurationObject) list
							.get(i);
					if (kriObj
							.isOfType(Unterzentrale.__ConfigurationObjectTypeOfKRI)) {
						_KRIList.add(KRI.getInstance(kriObj.getPid()));
					}
				}
			}
		}
		Unterzentrale.__Cache.put(this.getPID(), this);
	}

	@Override
	ConfigurationObjectType getType() {
		if (Unterzentrale.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance()
					.getDataModel();
			Unterzentrale.__ConfigurationObjectType = (ConfigurationObjectType) model
					.getType("typ.uz");

			Unterzentrale.__ConfigurationObjectTypeOfKRI = (ConfigurationObjectType) model
					.getType("typ.kri");
			Unterzentrale.__AtgGeraet = model.getAttributeGroup("atg.gerät");

		}
		return Unterzentrale.__ConfigurationObjectType;
	}

	@Override
	public String getTreeString() {
		return getName();
	}

	/**
	 * gibt ein Array aller Objekte zurück, die vom Typ typ.uz sind.
	 *
	 * @return Unterzentrale[]
	 */
	public static Unterzentrale[] getAll() throws DataModellMismatchException {
		final List list = LogosClientDavConnection.getInstance().getDataModel()
				.getType("typ.uz").getElements();
		final Unterzentrale[] retValue = new Unterzentrale[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new Unterzentrale((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	static public Unterzentrale getInstance(final ConfigurationObject confObj)
			throws DataModellMismatchException {
		final Unterzentrale uz = Unterzentrale.__Cache.get(confObj.getPid());
		if (uz != null) {
			return uz;
		} else {
			return new Unterzentrale(confObj);
		}
	}

	static public Unterzentrale getInstance(final String pid)
			throws DataModellMismatchException {
		final Object o = LogosClientDavConnection.getInstance().getDataModel()
				.getObject(pid);
		if ((o != null) && (o instanceof ConfigurationObject)) {
			return Unterzentrale.getInstance((ConfigurationObject) o);
		} else {
			return null;
		}
	}

}
