/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.util.Calendar;

import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * <p>
 * Channel zur Ausgabe von Daten in das BASt-Bestandsbandformat in der von
 * Sachsen-Anhalt verwendeten Version.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * <table border=1 align=center cellpadding=2 cellspacing=0 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkennzeichen</th> <th align=left>Bemerkung</th> </tr>
 * <tr> <td>&nbsp;</td> <td>korrekte, regelmäßige Werte</td> </tr> <tr>
 * <td>'</td> <td>alle anderen Werte</td> </tr> </table>
 *
 * <p> Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr
 * ausgegeben. </p>
 *
 * <p> siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt} </p>
 *
 */
public class DataChannelBASt_SA extends DataChannelBAStV1_X {

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle zaehlstelle)
	 * setZaehlstelle()} initialisiert werden muss.
	 */
	public DataChannelBASt_SA() {
		super();
		setCountOfGeschwKlassen(6);
	}

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle zaehlstelle)
	 * setZaehlstelle()} initialisiert werden muss zur Ausgabe der angegebenen
	 * SatzArt.
	 */
	public DataChannelBASt_SA(final int satzArt) {
		super(satzArt);
		setCountOfGeschwKlassen(6);
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link DataChannelBAStV1_X#processRecord(de.bsvrz.dav.daf.main.Dataset[], Calendar)}
	 * mit setZaehlStelle() initialisiert werden muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public DataChannelBASt_SA(final String fileName) throws ChannelException {
		super(fileName);
		setCountOfGeschwKlassen(6);
	}

	/**
	 * erzeugt den f&uuml;r eine Z&auml;hlstelle konstanten Zeilenanfang. Der
	 * Anfang jeder Zeile besteht aus TK-BlattNr, Z&auml;hlstellenNr.,
	 * L&auml;nderkennung.
	 */
	@Override
	protected String calculateLineHeader() {
		final StringBuffer lineHeader = new StringBuffer(10);
		lineHeader.append(_ZaehlStelle.getTK25AsString());
		lineHeader.append(_ZaehlStelle.getBastNummerAsString());
		final int iLaenderKennung = _ZaehlStelle.getBundesland().getKennung();
		if (iLaenderKennung < 10) {
			lineHeader.append('0');
		}
		lineHeader.append(iLaenderKennung);
		return lineHeader.toString();
	}

}
